
#include "global.h"



/***************************************************************************/
/** Process the commandline options                                       **/


void ProcessArguments(int Argc, char *Argv[])
{

char            *hdr_accept    = "";
char            *hdr_command   = "GET";
char            *hdr_from      = "";
char            *hdr_host      = "";
char            *hdr_cookie    = "";
char            *hdr_referer   = "";
char            *hdr_range     = "";
char            *hdr_agent     = "";

char            *path          = "";          /* e.g. 'files/this.zip'     */
char            *username      = "";          /* e.g. 'willy'              */
char            *password      = "";          /* e.g. 'free'               */
char            *b64_password  = "";          /* e.g. 'cmVuOmhlbGxvCg=='   */

char             escape        = 0;

int              t;



/***************************************************************************/
/** Process option given                                                  **/

while ((t = getopt(Argc, Argv, "def:a:C:F:h:c:r:R:u:v:P:")) != EOF) 
  {
  switch(t)
    {
    case 'd':    debug++; break;                     /* Turn on debugging              */
    case 'e':    escape=1; break;                    /* Escape chacters in url         */
    case 'f':    output_file=optarg; break;          /* Write data retrieved to file   */
    case 'C':    hdr_command=optarg; break;          /* Command to send, usually GET   */
    case 'F':    hdr_from=optarg; break;             /* From:                          */
    case 'R':    hdr_range=optarg; break;            /* Range: bytes=                  */
    case 'a':    hdr_accept=optarg; break;           /* Accept:                        */
    case 'c':    hdr_cookie=optarg; break;           /* Cookie:                        */
    case 'h':    hdr_host=optarg; break;             /* Host:                          */
    case 'r':    hdr_referer=optarg; break;          /* Referer:                       */
    case 'u':    hdr_agent=optarg; break;            /* User-Agent:                    */
    case 'P':    post_content=optarg; break;         /* POST message                   */
    case 'v':    if(optarg[0]=='r' || optarg[0]=='R')/* HTTP version                   */
                   hdr_version=0;
                   else
                   if(optarg[0]=='0') 
                     hdr_version=1;
                     else
                     if(optarg[0]=='1') 
                       hdr_version=2;
                 break;
    default:
    case '?':    Usage();                            /* Usage, does not return         */
    } /* end of switch() */
  } /* end of while() */

Argc -= optind;                                      /* Remove processed options       */
Argv += optind;
                


/***************************************************************************/
/** Check the arguments                                                   **/

if(Argc!=1) Usage();
if(strncmp("http://", Argv[0], 7) == 0)
  {
  service="80";
  ssl=0;
  DelNChar(Argv[0], 0, 7);
  }

if(strncmp("https://", Argv[0], 8) == 0)
  {
  service="443";
  ssl=1;
  DelNChar(Argv[0], 0, 8);
  }

/***************************************************************************/
/** Split up server and path                                              **/

host=Argv[0];
for(t=0; host[t]; t++) if(host[t]=='/') break;
if(host[t]=='\0') path="";
  else 
  path=&Argv[0][t+1];
host[t]='\0';



/***************************************************************************/
/** Get user and password if present                                      **/

for(t=0; host[t]; t++)
  if(host[t]=='@')
    {
    username=host;
    for(t=0; username[t]; t++) if(username[t]==':') break;
    if(username[t]=='\0') Usage();
    username[t]='\0';

    password=&username[t+1];
    for(t=0; password[t]; t++) if(password[t]=='@') break;
    if(password[t]=='\0') Usage();
    password[t]='\0';

    b64_password=Encode_Password(username, password);
    
    host=&password[t+1];
    break;
    }



/***************************************************************************/
/** Get port number if present                                            **/

for(t=0; host[t]; t++)
  if(host[t]==':') 
    {
    host[t]='\0';
    if(host[t+1]) service=&host[t+1];
    break;
    }
    
if(debug)
  fprintf(stderr, "-------------------------------------\n"
                  "Username: [%s]\n"
                  "Password: [%s]\n"
                  "B64_pass: [%s]\n"
                  "Host    : [%s]\n"
                  "Port    : [%s]\n"
                  "Path    : [%s]\n"
                  "Debug   : [%i]\n"
                  ,username
                  ,password
                  ,b64_password
                  ,host
                  ,service
                  ,path
                  ,debug
                  );


/***************************************************************************/
/** Construct request buffer                                              **/

if(post_content[0]) hdr_command="POST";

sprintf(request, "%s /%s", hdr_command, path);
switch(hdr_version)
  {
  case 0:    strcat(request, "\n");
             break;
  case 1:    strcat(request, " HTTP/1.0\n");
             break;
  case 2:    strcat(request, " HTTP/1.1\n");
             break;
  }

if(hdr_version==2 || hdr_host[0])
  {
  if(hdr_host[0])
    sprintf(temp_buffer, "Host: %s\n", hdr_host);
    else
    sprintf(temp_buffer, "Host: %s\n", host);
  strcat(request, temp_buffer);
  }

if(b64_password[0])
  {
  sprintf(temp_buffer, "Authorization: Basic %s\n", b64_password);
  strcat(request, temp_buffer);
  }

if(hdr_range[0])
  {
  sprintf(temp_buffer, "Range: bytes=%s\n", hdr_range);
  strcat(request, temp_buffer);
  }

if(hdr_accept[0])
  {
  sprintf(temp_buffer, "Accept: %s\n", hdr_accept);
  strcat(request, temp_buffer);
  }

if(hdr_from[0])
  {
  sprintf(temp_buffer, "From: %s\n", hdr_from);
  strcat(request, temp_buffer);
  }

if(hdr_referer[0])
  {
  sprintf(temp_buffer, "Referer: %s\n", hdr_referer);
  strcat(request, temp_buffer);
  }

if(hdr_cookie[0])
  {
  sprintf(temp_buffer, "Cookie: %s\n", hdr_cookie);
  strcat(request, temp_buffer);
  }

if(hdr_agent[0])
  {
  sprintf(temp_buffer, "User-Agent: %s\n", hdr_agent);
  strcat(request, temp_buffer);
  }

if(post_content[0])
  {
  sprintf(temp_buffer, "Content-type: application/x-www-form-urlencoded\nContent-length: %u\n\n%s", strlen(post_content), post_content);
  strcat(request, temp_buffer);
  }

strcat(request, "\n");

if(debug)
  fprintf(stderr, "-------------------------------------\n%s-----------------------------------\n", request);

}




