
#include "global.h"

/***************************************************************************\
**                                                                         **
**    ConnectSocket(...)                                                   **
**                                                                         **
**    Host      - Name or IP address of the target machine                 **
**    Service   - Name or number of the service (eg. www)                  **
**    Protocol  - Name of the protocol to use (either tcp or udp)          **
**                                                                         **
**    Returns a signed int, namely the socket handler.                     **
**                                                                         **
**                                                                         **
**    This function will exit the program with errorcode:                  **
**                                                                         **
**    1 - Specified wrong argument for the socket to connect to            **
**    2 - Network error                                                    **
**                                                                         **
\***************************************************************************/

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>  /* ip_addr */

signed int ConnectSocket(char *Host, char *Service, char *Protocol)
{
struct hostent      *phe;      /* pointer to host information entry    */
struct servent      *pse;      /* pointer to service information entry */
struct protoent     *ppe;      /* pointer to protocol information entry*/
struct sockaddr_in   sin;      /* an Internet endpoint address         */
signed int           s;        /* socket descriptor                    */
int                  type;     /* socket type                          */

/* clean the 'sin' struct */
bzero((char *)&sin, sizeof(sin));

/* define type Internet in 'sin' */
sin.sin_family = AF_INET;

/* map service name to port number */
if((pse=getservbyname(Service, Protocol))) 
  sin.sin_port = pse->s_port;
  else
  {
  if( (sin.sin_port=htons((u_short)atoi(Service))) == 0)
    {
    printf("\n Cannot get service number for [%s]\n\n", Service);
    exit(1);
    }
  }

/* map host name to IP address, allowing for dotted decimal */
if((phe=gethostbyname(Host)))
  bcopy(phe->h_addr, (char *)&sin.sin_addr, phe->h_length);
  else
	  {
  if( (sin.sin_addr.s_addr = inet_addr(Host)) == INADDR_NONE)
    {
    printf("\n Cannot get IP number for [%s]\n\n", Host);
    exit(1);
    }
  }

/* map protocol name to protocol number */
if( (ppe=getprotobyname(Protocol)) == 0)
  {
  printf("\n Wrong protocol specified [%s]\n\n", Protocol);
  exit(1);
  }

/* use protocol to choose a socket type */
if(strcmp(Protocol, "udp") == 0)
  type = SOCK_DGRAM;
  else
  type = SOCK_STREAM;

/* allocate a socket */
s=socket(PF_INET, type, ppe->p_proto);
if(s<0)
  {
  printf("\n Cannot create socket\n\n");
  exit(2);
  }

/* connect the socket */
if(connect(s, (struct sockaddr *)&sin, sizeof(sin)) < 0)
  {
  printf("\n Cannot connect to [%s:%s]\n\n", Host, Service);
  exit(2);
  }

return s;
}
