/***************************************************************************\
**                                                                         **
**  htdump.c                                                               **
**                                                                         **
**  Copyright 1999 by Ren Hoek (ren@arak.ptf.hro.nl)                       **
**                                                                         **
\***************************************************************************/



/***************************************************************************/
/** Defines                                                               **/

#define INTERN         1       /* Tie global variables to this object */



/***************************************************************************/
/** Includes                                                              **/

#include "global.h"

#if SSL_ENABLED
  #warning ----------------------------------
  #warning SSL ENABLED
  #warning ----------------------------------
  #include <openssl/ssl.h>
#endif





void main(int Argc, char *Argv[])
{

/***************************************************************************/
/** Declare variables and pointers                                        **/

unsigned int       fp                = STDOUT_FILENO;
signed int         socket            = 0;
unsigned int       t                 = 0;
unsigned int       s                 = 0;
unsigned int       r                 = 0;
char               scan_http_headers = 0;
char               line[256]         = "";


#if SSL_ENABLED
  SSL             *ssl_socket        = NULL;      /* Struct for SSL bookkeeping */
  SSL_CTX         *ctx               = NULL;
#endif        




/***************************************************************************/
/** Initialise global variables                                           **/

debug           = 0;
content_length  = 0;
response_code   = 0;
host            = "";          /* e.g. 'www.netscape.com'   */
service         = "80";        /* e.g. '8080'               */
output_file     = "";
ssl             = 0;           /* Going to use SSL or not?  */
hdr_version     = 1;           /* Standard on HTTP/1.0 */
post_content    = "";
post_type       = 0;




/***************************************************************************/
/** Process the commandline options and construct request buffer          **/

ProcessArguments(Argc, Argv);



/***************************************************************************/
/** Initialise SSL stuff                                                  **/

if(ssl)
  {
  #if SSL_ENABLED
    SSLeay_add_ssl_algorithms();
    ctx=SSL_CTX_new(SSLv23_client_method());
    if(ctx == NULL)
      {
      printf("\nError! Could not allocate memory for SSL CTX\n\n");
      exit(1);
      }
    ssl_socket=SSL_new(ctx);
  #else
    printf("\nError! SSL not compiled into htdump, can not use SSL\n\n");
    exit(1);
  #endif
  }


/***************************************************************************/
/** Open (SSL) socket                                                     **/

socket = ConnectSocket(host, service, "tcp");


#if SSL_ENABLED
  if(ssl)
    {
    SSL_set_fd(ssl_socket, socket);
    if(SSL_connect(ssl_socket) == -1)
      {
      printf("\nError! Could not SSL connect to server\n\n");
      exit(1);
      }
    if(debug) fprintf(stderr, "SSL connection using %s\n", SSL_get_cipher(ssl_socket));
    }
#endif



/***************************************************************************/
/** Write request                                                         **/

if(ssl)
  {
  #if SSL_ENABLED
    t=SSL_write(ssl_socket, request, strlen(request));
  #endif
  }
  else
  t=write(socket, request, strlen(request));

if(t==-1)  
  {
  printf("\nError! Could not write request to server\n\n");
  exit(1);
  }





/***************************************************************************/
/** Read response                                                         **/



if(output_file[0])
  {
  fp=open(output_file, O_WRONLY|O_CREAT);
  if(fp==-1)
   {
   printf("\nError opening file [%s]\n\n", output_file);
   perror("feep");
   exit(9);
   }
  }


if(hdr_version) scan_http_headers=1;
  else scan_http_headers=0;

r=RESPONSE_SIZE-1;

for(;;)
  {
  if(ssl)
    {
    #if SSL_ENABLED
      t = SSL_read (ssl_socket, response, r);
    #endif
    }
    else
    t = read(socket, response, r);

  if(t==-1)
    {
    printf("Error - Could not read from socket!");
    exit(1);
    }

  if(t==0)      /* End of what the server has to send */
    {
/*    printf("semi-Error - Did not get what we'd expect");*/
    break;
    }
    
   

  if(scan_http_headers)                                  /* Read headers */
    while( (s=sgets(line, 255, response)) )
      {
      t=t-s;     /* Response buffer minus what we read out of it */
 
      if(strncmp(line, "HTTP", 4)==0) response_code=atoi(line+9);

      if(strncmp(line, "Content-Length: ", 16)==0)
        {
        content_length=atoi(line+16);
        if(!content_length) content_length=-1;      /* Content-length == 0 */
        }

      if(debug) fprintf(stderr, "header: [%s]\n", line);      

      if(line[0]=='\0') 
        {
        if(debug)
          {
          fprintf(stderr, "-------------------------------------\n");
          if(content_length==-1) 
            fprintf(stderr, "No content!\n");
            else
            fprintf(stderr, "Content-length: %lu\n", content_length);
            fprintf(stderr, "Errorcode: %u\n"
                            "-------------------------------------\n"
                            ,response_code
                            );
          }
        scan_http_headers=0;
        break;
        }

      if(t==0) break;   /* Get new load of data */
      /* BUG, I think if receive thingie is short, and
         data gets read in once more, the current header
         is split.. */
      } /* End of scan_headers routine */

  if(content_length)       /* Keep track of how many data we got to go */
    {
    if(content_length==-1) break;
    content_length=content_length-t;
    if(content_length<RESPONSE_SIZE) r=content_length;
      else r=RESPONSE_SIZE;                              /* Read last chunk */
    }
    else r=RESPONSE_SIZE;

  if(write(fp, response, t)==-1)                 /* Write data to screen */
    {
    printf("\nError writing to file!\n\n");
    exit(9);
    }
  } /* End of for(;;) loop */


/***************************************************************************/
/** Close socket                                                          **/

if(output_file[0]) close(fp);

#if SSL_ENABLED
  if(ssl) SSL_shutdown (ssl_socket);  /* send SSL/TLS close_notify */
#endif

close(socket);

#if SSL_ENABLED
  SSL_free (ssl_socket);
  SSL_CTX_free (ctx);
#endif

} /* end of main() */


